@@ -53,14 +53,13 @@ struct YawpConfigDialog::Private
 {
 	KConfigDialog              * pConfigDlg;
 	WeatherServiceModel        * pWeatherModel;
-	bool                         bCityModelChanged;
-	bool                         bCitySettingsChanged;
-	bool                         bUnitsChanged;
+	bool                         bWeatherDataUpdateRequired;
 
-// This is used to map localized theme names to the english names
+	// This is used to map localized theme names to the english names
 	QMap <int, QString>          theme_map;
 	
 	Yawp::Storage              * pStorage;
+	QPointer<DlgAddCity>         pCitySearchDialog;
 };
 
 YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent, Yawp::Storage * pStorage )
@@ -69,14 +68,13 @@ YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent, Yawp::Storage * pSto
 {
 	d->pStorage = pStorage;
 	
-	d->bCityModelChanged = false;
-    d->bCitySettingsChanged = false;
+	d->bWeatherDataUpdateRequired = false;
 
 	d->pConfigDlg = parent;
 	d->pWeatherModel = new WeatherServiceModel(pStorage, this);
 	d->pWeatherModel->setObjectName("ConfigModel");
 
-// Fill theme map
+	// Fill theme map
 	d->theme_map.insert(0, "default");
 	d->theme_map.insert(1, "purple");
 	d->theme_map.insert(2, "green");
@@ -87,19 +85,22 @@ YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent, Yawp::Storage * pSto
 	d->theme_map.insert(7, "funky");
 	d->theme_map.insert(8, "naked");
 
-	QWidget * locationsPage	= new QWidget( parent );
-	QWidget * settingsPage	= new QWidget( parent );
-	QWidget * panelPage	= new QWidget( parent );
-	QWidget * themePage	= new QWidget( parent );
+	QWidget * locationsPage = new QWidget(parent);
+	QTabWidget * settingsPage = new QTabWidget(parent);
+	QWidget * desktopPage = new QWidget(parent);
+	QTabWidget * panelPage = new QTabWidget(parent);
+	QWidget * themePage = new QWidget(parent);
 
 	Ui::LocationsPage::setupUi(locationsPage);
 	Ui::SettingsPage::setupUi(settingsPage);
+	Ui::DesktopPage::setupUi(desktopPage);
 	Ui::PanelPage::setupUi(panelPage);
 	Ui::ThemePage::setupUi(themePage);
 
 	parent->setButtons(KDialog::Ok | KDialog::Cancel | KDialog::Apply);
 	parent->addPage(locationsPage, i18n("Locations"), QLatin1String("preferences-desktop-locale"));
 	parent->addPage(settingsPage,  i18n("Settings"),  QLatin1String("preferences-system"));
+	parent->addPage(desktopPage,   i18n("Desktop"),   QLatin1String("preferences-desktop-display"));
 	parent->addPage(panelPage,     i18n("Panel"),     QLatin1String("preferences-system-windows"));
 	parent->addPage(themePage,     i18n("Theme"),     QLatin1String("plasma"));
 
@@ -114,7 +115,7 @@ YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent, Yawp::Storage * pSto
 	cityTableView->setColumnWidth( 1, 100 );
 	cityTableView->setColumnWidth( 2, 150 );
 
-	connect(btnAddCity,     SIGNAL(clicked()),     this, SLOT(addCity()));
+	connect(btnAddCity,     SIGNAL(clicked()),     this, SLOT(showCitySearchDialog()));
 	connect(btnDeleteCity,  SIGNAL(clicked()),     this, SLOT(deleteCity()));
 	connect(cityTableView,  SIGNAL(clicked(const QModelIndex &)),   this, SLOT(locationSelected(const QModelIndex &)));
 	connect(btnMoveUp,      SIGNAL(clicked()),     this, SLOT(moveSelectedCityUp()));
@@ -125,9 +126,12 @@ YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent, Yawp::Storage * pSto
 	btnDeleteCity->setIcon( KIcon("list-remove") );
 	btnMoveUp->setIcon( KIcon("go-up") );
 	btnMoveDown->setIcon( KIcon("go-down") );
+	btnSetTimeZone->setIcon( KIcon("clock") );
 
 	//--- THEME PAGE ---
 	//
+	btnCustomThemeFile->setIcon( KIcon("document-open") );
+	  
 	connect(checkBoxCustomTheme,		SIGNAL(stateChanged(int)),	this, SLOT(changeThemeState(int)));
 	connect(checkBoxCustomTheme,		SIGNAL(clicked()),		this, SLOT(enableYawpBackground()));
 	connect(checkBoxUseCustomBackground,	SIGNAL(clicked()),		this, SLOT(enableYawpBackground()));
@@ -135,7 +139,7 @@ YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent, Yawp::Storage * pSto
 	connect(checkBoxUseCustomColor,		SIGNAL(stateChanged(int)),	this,  SLOT(enableCustomFontColorOptions(int)));
 
 	//--- SETUP UNIT SYSTEM ---
-#if (KDE_VERSION_MINOR == 3 && KDE_VERSION_RELEASE >= 70) || KDE_VERSION_MINOR == 4
+#if KDE_IS_VERSION(4,3,70)
 	comboTemperature->insertItem( 0, i18n("Celsius"),	QVariant(KUnitConversion::Celsius) );
 	comboTemperature->insertItem( 1, i18n("Fahrenheit"),	QVariant(KUnitConversion::Fahrenheit) );
 
@@ -149,7 +153,11 @@ YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent, Yawp::Storage * pSto
 	comboPressure->insertItem( 1, i18n("Inches HG"),	QVariant(KUnitConversion::InchesOfMercury) );
 	comboPressure->insertItem( 2, i18n("Millibars"),	QVariant(KUnitConversion::Millibar) );
 	comboPressure->insertItem( 3, i18n("Hectopascals"),	QVariant(KUnitConversion::Hectopascal) );
+#if KDE_IS_VERSION(4,4,92)
+	comboPressure->insertItem( 4, i18n("Millimeters HG"),	QVariant(KUnitConversion::MillimetersOfMercury) );
+#else
 	comboPressure->insertItem( 4, i18n("Torr (Millimeter HG)"),	QVariant(KUnitConversion::Torr) );
+#endif
 
 	comboDistance->insertItem( 3, i18n("Kilometers"),	QVariant(KUnitConversion::Kilometer) );
 	comboDistance->insertItem( 4, i18n("Miles"),		QVariant(KUnitConversion::Mile) );
@@ -195,7 +203,17 @@ YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent, Yawp::Storage * pSto
 	//
 	connect(checkCompactPanelLayout, SIGNAL(toggled(bool)), this, SLOT(enableCompactPanelLayout(bool)));
 	connect(radioExtendedTooltip, SIGNAL(toggled(bool)), this, SLOT(enableExtendedTooltipOptions(bool)));
-
+	
+	QButtonGroup * grpPanelWeather = new QButtonGroup(this);
+	grpPanelWeather->addButton(radioActualTemp);
+	grpPanelWeather->addButton(radioActualIcon);
+	grpPanelWeather->addButton(radioActualBoth);
+	
+	QButtonGroup * grpPanelForecasts = new QButtonGroup(this);
+	grpPanelForecasts->addButton(radioForecastTemp);
+	grpPanelForecasts->addButton(radioForecastIcon);
+	grpPanelForecasts->addButton(radioForecastBoth);
+	
 	//--- RESIZE AND SET MINIMUM SIZE ---
 	QSize minSize( parent->sizeHint() );
 	minSize.setWidth( 650 );
@@ -207,11 +225,64 @@ YawpConfigDialog::YawpConfigDialog( KConfigDialog * parent, Yawp::Storage * pSto
 		KMessageBox::sorry( parent, i18n("Your system does not have any weather-engine. %1 is useless without at least one weatherengine.", QLatin1String(YAWP_NAME)) );
 		btnAddCity->setEnabled( false );
 	}
+	
+	//--- TRIGGER SLOT MODIFIED WHEN CONFIG-DIALOG HAS BEEN CHANGED ---
+	//
+	//--- Settings Page - tab unit systems ---
+	connect(comboDistance, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(comboPressure, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(comboTemperature, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(comboSpeed, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	
+	//--- Settings Page - tab animation ---
+	connect(comboPageAnimation, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(comboDayformatAnimation, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(comboDetailAnimation, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(comboIconAnimation, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(sliderAnimationDuration, SIGNAL(valueChanged(int)), this, SLOT(settingsChanged()));
+	
+	//--- Settings Page - tab update behaviour ---
+	connect(comboInterval,  SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(spinboxStartDelay, SIGNAL(valueChanged(int)), this, SLOT(settingsChanged()));
+	connect(groupBoxTraverseLocationsPeriodically, SIGNAL(clicked()), this, SLOT(settingsChanged()));
+	connect(spinTraverseLocationsTimeout, SIGNAL(valueChanged(int)), this, SLOT(settingsChanged()));
+	
+	//--- Desktop Page ---
+	connect(radioTraditionalDesktopPainter, SIGNAL(clicked()), this, SLOT(settingsChanged()));
+	connect(radioExtendedDesktopPainter, SIGNAL(clicked()), this, SLOT(settingsChanged()));
+	
+	//--- Panel Page ---
+	connect(grpPanelWeather, SIGNAL(buttonClicked(int)), this, SLOT(settingsChanged()));
+	connect(grpPanelForecasts, SIGNAL(buttonClicked(int)), this, SLOT(settingsChanged()));
+	connect(comboForecastDays, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(checkCompactPanelLayout, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
+	connect(checkInteractivePanelWeatherIcons, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
+	connect(radioSimpleTooltip, SIGNAL(clicked()), this, SLOT(settingsChanged()));
+	connect(radioExtendedTooltip, SIGNAL(clicked()), this, SLOT(settingsChanged()));
+	connect(checkTooltipPreview, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
+	connect(checkTooltipSateliteMap, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
+	connect(checkTooltipThemeBackground, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
+
+	connect(radioTraditionalPopupPainter, SIGNAL(clicked()), this, SLOT(settingsChanged()));
+	connect(radioExtendedPopupPainter, SIGNAL(clicked()), this, SLOT(settingsChanged()));
+
+	//--- Theme Page ---
+	connect(comboTheme, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(checkBoxCustomTheme, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
+	connect(checkBoxUseCustomBackground, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
+	connect(checkBoxUseCustomColor, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
+	connect(comboColor, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(comboLowerColor, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(comboShadowsColor, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(comboForecastTextColor, SIGNAL(currentIndexChanged(int)), this, SLOT(settingsChanged()));
+	connect(checkBoxDisableTextShadows, SIGNAL(stateChanged(int)), this, SLOT(settingsChanged()));
 }
 
 YawpConfigDialog::~YawpConfigDialog()
 {
 	delete d->pWeatherModel;
+	if (d->pCitySearchDialog)
+		delete d->pCitySearchDialog;
 	delete d;
 }
 
@@ -236,24 +307,17 @@ YawpConfigDialog::weatherModel() const
 }
 
 bool
-YawpConfigDialog::cityModelChanged() const
+YawpConfigDialog::weatherDataUpdateRequired() const
 {
-	return d->bCityModelChanged;
+	return d->bWeatherDataUpdateRequired;
 }
 
-bool
-YawpConfigDialog::citySettingsChanged() const
+void
+YawpConfigDialog::resetChanges()
 {
-	return d->bCitySettingsChanged;
+	d->bWeatherDataUpdateRequired = false;
 }
 
-bool
-YawpConfigDialog::unitsChanged() const
-{
-	return d->bUnitsChanged;
-}
-
-
 void
 YawpConfigDialog::setData( const Yawp::ConfigData * data )
 {
@@ -268,8 +332,11 @@ YawpConfigDialog::setData( const Yawp::ConfigData * data )
 	//--- UPDATE INTERVAL ---
 	iIdx = comboInterval->findData( data->iUpdateInterval );
 	comboInterval->setCurrentIndex( iIdx >= 0 ? iIdx : 0 );
-
+	
 	spinboxStartDelay->setValue( data->iStartDelay );
+	
+	groupBoxTraverseLocationsPeriodically->setChecked( data->bTraverseLocationsPeriodically );
+	spinTraverseLocationsTimeout->setValue( data->iTraverseLocationTimeout );
 
 	//--- UNIT SYSTEM ---
 	iIdx = comboTemperature->findData( data->temperatureSystem );
@@ -297,6 +364,8 @@ YawpConfigDialog::setData( const Yawp::ConfigData * data )
 	//--- THEME PAGE ---
 	//
 	checkBoxCustomTheme->setCheckState( data->bUseCustomTheme ? Qt::Checked : Qt::Unchecked );
+	checkInteractivePanelWeatherIcons->setCheckState( data->bUseInteractivePanelWeatherIcons ? Qt::Checked : Qt::Unchecked );
+	
 	changeThemeState( checkBoxCustomTheme->checkState() );
 
 	comboTheme->setCurrentIndex( d->theme_map.key(data->sBackgroundName) );
@@ -309,27 +378,52 @@ YawpConfigDialog::setData( const Yawp::ConfigData * data )
 	comboColor->setColor( data->fontColor );
 	comboLowerColor->setColor( data->lowFontColor );
 	comboShadowsColor->setColor( data->shadowsFontColor );
+	comboForecastTextColor->setColor( data->forecastTextColor );
 
 	//--- TEXT SHADOWS ---
 	checkBoxDisableTextShadows->setCheckState( data->bDisableTextShadows ? Qt::Checked : Qt::Unchecked );
 
+	//--- DESKTOP PAGE ---
+	switch (data->desktopPainterType)
+	{
+		case Yawp::ExtendedDesktopPainter:
+			radioExtendedDesktopPainter->setChecked(true);
+			break;
+		default:
+			radioTraditionalDesktopPainter->setChecked(true);
+			break;
+	}
+	
 	//--- PANEL PAGE ---
 	//
 	//--- current day settings ---
-	if( data->todaysWeatherPanelFormat == Yawp::PanelTemperature )
-		radioActualTemp->setChecked( true );
-	else if( data->todaysWeatherPanelFormat ==  Yawp::PanelIcon )
-		radioActualIcon->setChecked( true );
-	else
-		radioActualBoth->setChecked( true );
+	switch( data->todaysWeatherPanelFormat )
+	{
+		case Yawp::PanelTemperature:
+			radioActualTemp->setChecked( true );
+			break;
+		case Yawp::PanelIcon:
+			radioActualIcon->setChecked( true );
+			break;
+		default:
+			radioActualBoth->setChecked( true );
+			break;
+	}
 
 	//--- forecast days settings ---
-	if( data->forecastWeatherPanelFormat == Yawp::PanelTemperature )
-		radioForecastTemp->setChecked( true );
-	else if( data->forecastWeatherPanelFormat == Yawp::PanelIcon )
-		radioForecastIcon->setChecked( true );
-	else
-		radioForecastBoth->setChecked( true );
+	switch( data->forecastWeatherPanelFormat )
+	{
+		case Yawp::PanelTemperature:
+			radioForecastTemp->setChecked( true );
+			break;
+		case Yawp::PanelIcon:
+			radioForecastIcon->setChecked( true );
+			break;
+		default:
+			radioForecastBoth->setChecked( true );
+			break;
+	}
+	
 	comboForecastDays->setCurrentIndex( data->iPanelForecastDays );
 
 	checkCompactPanelLayout->setChecked( data->bUseCompactPanelLayout );
@@ -339,6 +433,7 @@ YawpConfigDialog::setData( const Yawp::ConfigData * data )
 		radioExtendedTooltip->setChecked( true );
 	else
 		radioSimpleTooltip->setChecked( true );
+	
 	if( data->extendedTooltipOptions.testFlag( Yawp::SatellitePage ) )
 		checkTooltipSateliteMap->setChecked( true );
 	if( data->extendedTooltipOptions.testFlag( Yawp::PreviewPage ) )
@@ -346,6 +441,19 @@ YawpConfigDialog::setData( const Yawp::ConfigData * data )
 	if( data->extendedTooltipOptions.testFlag( Yawp::ThemeBackground ) )
 		checkTooltipThemeBackground->setChecked( true );
 	enableExtendedTooltipOptions( radioExtendedTooltip->isChecked() );
+	
+	switch(data->popupPainterType)
+	{
+		case Yawp::ExtendedDesktopPainter:
+			radioExtendedPopupPainter->setChecked(true);
+			break;
+		default:
+			radioTraditionalPopupPainter->setChecked(true);
+			break;
+	}
+	
+	//--- unset the apply button ---
+	settingsChanged(false);
 }
 
 void
@@ -362,7 +470,12 @@ YawpConfigDialog::getData( Yawp::ConfigData * data )
 	//--- UPDATE INTERVAL ---
 	int iIdx = comboInterval->currentIndex();
 	data->iUpdateInterval = comboInterval->itemData( iIdx ).toInt();
+	
 	data->iStartDelay = spinboxStartDelay->value();
+	
+	data->bTraverseLocationsPeriodically = groupBoxTraverseLocationsPeriodically->isChecked();
+	data->iTraverseLocationTimeout = spinTraverseLocationsTimeout->value();
+
 
 	//--- UNIT SYSTEM ---
 	int iTempIdx  = comboTemperature->currentIndex();
@@ -370,20 +483,11 @@ YawpConfigDialog::getData( Yawp::ConfigData * data )
 	int iPressureIdx = comboPressure->currentIndex();
 	int iDistanceIdx = comboDistance->currentIndex();
 
-	d->bUnitsChanged = false;
-	if ((int)data->temperatureSystem != comboTemperature->itemData( iTempIdx ).toInt() ||
-	    (int)data->speedSystem       != comboSpeed->itemData( iSpeedIdx ).toInt() ||
-	    (int)data->pressureSystem    != comboPressure->itemData( iPressureIdx ).toInt() ||
-	    (int)data->distanceSystem    != comboDistance->itemData( iDistanceIdx ).toInt() )
-	{
-		d->bUnitsChanged = true;
-	}
 	data->temperatureSystem = (YAWP_TEMPERATURE_UNIT)comboTemperature->itemData( iTempIdx ).toInt();
 	data->speedSystem       = (YAWP_SPEED_UNIT)comboSpeed->itemData( iSpeedIdx ).toInt();
 	data->pressureSystem    = (YAWP_PRESSURE_UNIT)comboPressure->itemData( iPressureIdx ).toInt();
 	data->distanceSystem    = (YAWP_DISTANCE_UNIT)comboDistance->itemData( iDistanceIdx ).toInt();
 
-
 	iIdx = comboPageAnimation->currentIndex();
 	data->pageAnimation = (PageAnimator::Transition)comboPageAnimation->itemData( iIdx ).toInt();
 	iIdx = comboDayformatAnimation->currentIndex();
@@ -399,8 +503,8 @@ YawpConfigDialog::getData( Yawp::ConfigData * data )
 	data->bUseCustomTheme = (checkBoxCustomTheme->checkState() == Qt::Checked ? true : false);
 	data->sBackgroundName = d->theme_map[comboTheme->currentIndex()];
 	data->bUseCustomThemeBackground = (checkBoxUseCustomBackground->checkState() == Qt::Checked ? true : false);
-	data->sCustomThemeFile = editCustomThemeFile->text();
-
+	data->sCustomThemeFile = editCustomThemeFile->originalText();
+	
 	//--- USE CUSTOM COLOR ---
 	data->bUseCustomFontColor = (checkBoxUseCustomColor->checkState() == Qt::Checked ? true : false);
 	if( data->bUseCustomFontColor )
@@ -408,11 +512,18 @@ YawpConfigDialog::getData( Yawp::ConfigData * data )
 		data->fontColor = comboColor->color();
 		data->lowFontColor = comboLowerColor->color();
 		data->shadowsFontColor = comboShadowsColor->color();
+		data->forecastTextColor = comboForecastTextColor->color();
 	}
 
 	//--- TEXT SHADOWS ---
 	data->bDisableTextShadows = (checkBoxDisableTextShadows->checkState() == Qt::Checked ? true : false);
 
+	//--- DESKTOP PAGE ---
+	if (radioTraditionalDesktopPainter->isChecked())
+		data->desktopPainterType = Yawp::DesktopPainter;
+	else //if (radioExtendedDesktopPainter->isChecked())
+		data->desktopPainterType = Yawp::ExtendedDesktopPainter;
+	
 	//--- PANEL PAGE ---
 	//
 	//--- current day settings ---
@@ -433,6 +544,7 @@ YawpConfigDialog::getData( Yawp::ConfigData * data )
 	data->iPanelForecastDays = comboForecastDays->currentIndex();
 
 	data->bUseCompactPanelLayout = checkCompactPanelLayout->isChecked();
+	data->bUseInteractivePanelWeatherIcons = checkInteractivePanelWeatherIcons->isChecked();
 	data->bUseExtendedTooltip = radioExtendedTooltip->isChecked();
 	data->extendedTooltipOptions = 0;
 	if( checkTooltipSateliteMap->isChecked() )
@@ -441,6 +553,11 @@ YawpConfigDialog::getData( Yawp::ConfigData * data )
 		data->extendedTooltipOptions |= Yawp::PreviewPage;
 	if( checkTooltipThemeBackground->isChecked() )
 		data->extendedTooltipOptions |= Yawp::ThemeBackground;
+	
+	if (radioTraditionalPopupPainter->isChecked())
+		data->popupPainterType = Yawp::DesktopPainter;
+	else //if (radioExtendedPopupPainter->isChecked())
+		data->popupPainterType = Yawp::ExtendedDesktopPainter;
 }
 
 void
@@ -459,9 +576,12 @@ YawpConfigDialog::enableCustomFontColorOptions(int state)
 	label_comboColor->setEnabled(bEnabled);
 	label_comboLowerColor->setEnabled(bEnabled);
 	label_comboShadowsColor->setEnabled(bEnabled);
+	label_comboForecastTextColor->setEnabled(bEnabled);
+	
 	comboColor->setEnabled(bEnabled);
 	comboLowerColor->setEnabled(bEnabled);
 	comboShadowsColor->setEnabled(bEnabled);
+	comboForecastTextColor->setEnabled(bEnabled);
 }
 
 void
@@ -474,9 +594,9 @@ void
 YawpConfigDialog::selectCustomThemeFile()
 {
 	QString sDir;
-	if( !editCustomThemeFile->text().isEmpty() )
+	if( !editCustomThemeFile->originalText().isEmpty() )
 	{
-		QFileInfo info( editCustomThemeFile->text() );
+		QFileInfo info( editCustomThemeFile->originalText() );
 		sDir = info.absolutePath();
 	}
 	else
@@ -486,8 +606,14 @@ YawpConfigDialog::selectCustomThemeFile()
 		KUrl(sDir),
 		"*.svg *.svgz|Scalable Vector Graphics",
 		d->pConfigDlg );
+	
 	if( !filename.isEmpty())
+	{
 		editCustomThemeFile->setText(filename);
+		
+		//--- notify config dialog, that settings has been changed ---
+		settingsChanged();
+	}
 }
 
 void
@@ -516,29 +642,45 @@ YawpConfigDialog::enableExtendedTooltipOptions( bool bEnabled )
 }
 
 void
-YawpConfigDialog::addCity()
+YawpConfigDialog::showCitySearchDialog()
 {
-	DlgAddCity dlg( d->pStorage, d->pConfigDlg );
-	if( dlg.exec() == QDialog::Accepted )
+	if (d->pCitySearchDialog == 0)
 	{
-		CityWeather cityInfo = dlg.getSelectedCity();
-		int idx = d->pWeatherModel->addCity( cityInfo );
+		d->pCitySearchDialog = new DlgAddCity(d->pStorage, d->pConfigDlg);
+		d->pCitySearchDialog->setModal(false);
+		connect(d->pCitySearchDialog, SIGNAL(citySelected(CityWeather *)), this, SLOT(addCity(CityWeather *)));
+	}
+	d->pCitySearchDialog->show();
+	d->pCitySearchDialog->raise();
+	d->pCitySearchDialog->activateWindow();
+}
+
+void
+YawpConfigDialog::addCity(CityWeather * cityInfo)
+{
+	if (cityInfo != 0)
+	{
+		int idx = d->pWeatherModel->addCity( *cityInfo );
 
 		if( idx >= 0 )
 		{
 			cityTableView->setCurrentIndex( d->pWeatherModel->index(idx, 0, QModelIndex()) );
 			updateLocationButtons();
-			d->bCitySettingsChanged = true;
-			d->bCityModelChanged = true;
+			d->bWeatherDataUpdateRequired = true;
+			
+			//--- notify config dialog, that settings has been changed ---
+			settingsChanged();
 		}
 		else
 		{
 			QMessageBox::information(
 				d->pConfigDlg,
 				i18n("Error"),
-				i18n("City %1 already exists in the list and can not be added again.", cityInfo.city()) );
+				i18n("City %1 already exists in the list and can not be added again.", cityInfo->city()) );
 		}
+		delete cityInfo;
 	}
+	
 }
 
 void
@@ -561,7 +703,9 @@ YawpConfigDialog::deleteCity()
 	{
 		d->pWeatherModel->removeRows( index.row(), 1 );
 		updateLocationButtons();
-		d->bCityModelChanged = true;
+		
+		//--- notify config dialog, that settings has been changed ---
+		settingsChanged();
 	}
 }
 
@@ -589,9 +733,10 @@ YawpConfigDialog::slotSetTimeZone()
 		cityInfo->setTimeZone( dlg.selectedTimeZone() );
 		if( cityInfo->countryCode().isEmpty() )
 			cityInfo->setCountryCode( cityInfo->timeZone().countryCode() );
-		d->bCitySettingsChanged = true;
-		d->bCityModelChanged = true;
-	}	
+		
+		//--- notify config dialog, that settings has been changed ---
+		settingsChanged();
+	}
 }
 
 void
@@ -647,6 +792,15 @@ YawpConfigDialog::moveSelectedCity( int offset )
 		index = index.sibling(index.row()+offset, 0);
 		cityTableView->setCurrentIndex( index );
 		locationSelected( index );
-		d->bCityModelChanged = true;
+		
+		//--- notify config dialog, that settings has been changed ---
+		settingsChanged();
 	}
 }
+
+void
+YawpConfigDialog::settingsChanged(bool changed)
+{
+	d->pConfigDlg->enableButton(KDialog::Ok, changed);
+	d->pConfigDlg->enableButton(KDialog::Apply, changed);
+}
