@@ -35,9 +35,9 @@
 
 //--- KDE4 ---
 #include <KDateTime>
-#include <KGlobalSettings>
 #include <KTimeZones>
 #include <KSystemTimeZones>
+#include <Plasma/Theme>
 
 #if (KDE_VERSION_MINOR == 3 && KDE_VERSION_RELEASE >= 70) || KDE_VERSION_MINOR == 4
 	#include <KUnitConversion/Value>
@@ -65,7 +65,7 @@ struct WeatherDataProcessor::Private
 
 	bool                            requestTimeZone;
 
-	QHash<Yawp::DetailsProperty, short> vDetailsPropertyMap;
+	QList<Yawp::DetailsProperty>    vDetailsProperty;
 
 	bool initIconMap(const QString & resource);
 
@@ -73,6 +73,9 @@ struct WeatherDataProcessor::Private
 	 *  In this cases the function returns false, otherwise true.
 	 */
 	bool checkStringValue( const QString & value ) const;
+	
+	QString createWindShortText(const YawpWeather & weather) const;
+	QString createPressureText(const YawpWeather & weather) const;
 
 	/** Parse a string that contains a float number.
 	 *  Returns FLT_MAX when parsing fails.
@@ -88,6 +91,8 @@ struct WeatherDataProcessor::Private
 	WeatherUtils::Unit convertSpeed2Distance( WeatherUtils::Unit fromSpeed ) const;
 #endif
 
+	QTime getTime(const QString & dateTime) const;
+	
 	void setUVValues( YawpWeather & weather, const QString & sUVIndex, const QString & sUVRating ) const;
 
 	bool isNightTime( QString & sShortDay ) const;
@@ -98,13 +103,6 @@ struct WeatherDataProcessor::Private
 	QString createPropertyText( const Yawp::DetailsProperty, const YawpDay & pDay, const YawpWeather & pWeather ) const;
 	void createPropertyStringList( const YawpDay & day, YawpWeather & weather ) const;
 	QSizeF doTextLayout( QTextLayout & textLayout, qreal dMaxWidth, qreal dOffset ) const;
-
-	inline void
-	addPropertyItem( QMap<short, Yawp::DetailsProperty> & vMap, Yawp::DetailsProperty property ) const
-	{
-		if( vDetailsPropertyMap.contains(property) )
-			vMap[ vDetailsPropertyMap.value(property) ] = property;
-	}
 };
 
 WeatherDataProcessor::WeatherDataProcessor( const Yawp::Storage * pStorage )
@@ -182,7 +180,23 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 
 	//--- Get weather forecast information ---
 	cityInfo.setCredit( data.value("Credit").toString() );
-	cityInfo.setCreditURL( data.value("Credit Url").toString() );
+	
+	//-- This is a stupid workaround ---
+	// ION BBC Weather Engine is returning the url to the weather forecast of current city
+	// Our own ION's accuweather and wunderground is following this example since version 0.4.0
+	// ION "Environment Canada" and "NOAA" does not returning "Credit Url" at all, probably because the provider
+	// does not support the information to send the url to the weather forecast
+	// Our own ION Google is following the example of ION "Environment Canada" and "NOAA"
+	// But ION wetter.com is returning the base url to the weather provider.
+	// Up to KDE version 4.2 all ION's was returning the base url to the weather provider,
+	// but at one point this has been changed and probably KDE-Developer forgot about ION wetter.com
+	//... at least I think so...
+	// Therefore we just check whether Credit Url contains a path or not - in the second case we suppose
+	// it is just the base url to the weather provider. In this case we do not use the credit Url.
+	KUrl creditUrl(data.value("Credit Url").toString());
+	if (creditUrl.isValid() && creditUrl.hasPath())
+		cityInfo.setCreditUrl(data.value("Credit Url").toString());
+	
 	int iForecastDays = data.value("Total Weather Days", "0").toInt();
 	QDate currDate;	// date of current day
 	int iTodayDays(0);
@@ -213,8 +227,8 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 		    vTokens.at(0).compare( i18n("Tonight"), Qt::CaseInsensitive) == 0 ||
 		    vTokens.at(0).startsWith( "Nite", Qt::CaseInsensitive) ||        // <--- envcan specific
 		    vTokens.at(0).startsWith( i18n("Nite"), Qt::CaseInsensitive) ||  // <--- envcan specific
-		    vTokens.at(0).startsWith( "Night", Qt::CaseInsensitive) ||       // <--- wettercom specific
-		    vTokens.at(0).startsWith( i18n("Night"), Qt::CaseInsensitive) )  // <--- wettercom specific
+		    vTokens.at(0).startsWith( "night", Qt::CaseInsensitive) ||       // <--- wettercom specific
+		    vTokens.at(0).startsWith( i18n("night"), Qt::CaseInsensitive) )  // <--- wettercom specific
 		{
 			dTracing() << "Adding Tonight for " << vTokens;
 			iTodayDays += 1;
@@ -233,7 +247,7 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 		}
 		else if( !currDate.isValid() )
 		{
-			dTracing() << "Date is not falid -> check " << vTokens;
+			dTracing() << "Date is not valid -> check " << vTokens;
 			QString sShortDay( vTokens.at(0) );
 			bool bNightTime = d->isNightTime( sShortDay );
 
@@ -295,8 +309,7 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 
 	//--- Set the sun for all forecast days - this is an accuweather specific key. ---
 	for( int iDay = 1; iDay < cityInfo.days().count(); ++iDay )
-		setForecastSun( *cityInfo.days().at(iDay),
-					    data.value( QString("Forecast Sun %1").arg(iDay) ).toString() );
+		setForecastSun( *cityInfo.days().at(iDay), data.value( QString("Forecast Sun %1").arg(iDay) ).toString() );
 
 	/***  Add the current weather informations to the first day ***
 	*/
@@ -312,20 +325,9 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 		pDay = cityInfo.days().at(0);
 
 	/*** PARSE SUNRISE AND SUNSET ***/
-	KDateTime sunTime;
-	sunTime = KDateTime::fromString(data.value("Sunrise At").toString(), "%A %B %e, %Y at %H:%M %Z", KSystemTimeZones::timeZones(), true);
-	if( sunTime.isValid() )
-		pDay->setSunrise( sunTime.time() );
-	else
-		pDay->setSunrise( QTime::fromString(data.value("Sunrise At").toString(), "h:m") );
-
-	sunTime = KDateTime::fromString(data.value("Sunset At").toString(), "%A %B %e, %Y at %H:%M %Z", KSystemTimeZones::timeZones(), true);
-	if( sunTime.isValid() )
-		pDay->setSunset( sunTime.time() );
-	else
-		pDay->setSunset(  QTime::fromString(data.value("Sunset At").toString(), "h:m") );
-
-
+	pDay->setSunrise( d->getTime(data.value("Sunrise At").toString()) );
+	pDay->setSunset( d->getTime(data.value("Sunset At").toString()) );
+	
 	YawpWeather & weather
 		= (pDay->hasNightValues() && !cityInfo.isDayTime(pDay) ? pDay->nightWeather() : pDay->weather());
 
@@ -333,8 +335,8 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 		weather.setCurrentIconName( mapConditionIcon( data.value("Condition Icon").toString() ) );
 	if( data.contains("Temperature") )
 		weather.setCurrentTemperature( d->convertTemp(  data.value("Temperature").toString(), fromTempSystem ) );
-	if( data.contains("Wind Speed") )
-		weather.setWindSpeed( d->convertSpeed( data.value("Wind Speed").toString(),  fromSpeedSystem ) );
+	if( data.contains("RealFeel Temperature") )
+		weather.setCurrentRealFeelTemperature( d->convertTemp( data.value("RealFeel Temperature").toString(), fromTempSystem ) );
 	if( data.contains("Dewpoint") )
 		weather.setDewpoint( d->convertTemp( data.value("Dewpoint").toString(), fromDewTempSystem ) );
 	if( data.contains("Visibility") )
@@ -345,9 +347,18 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 	if( d->checkStringValue( sTmp ) )
 		weather.setPressureTendency( sTmp );
 
+	if( data.contains("Wind Speed") )
+		weather.setWindSpeed( d->convertSpeed( data.value("Wind Speed").toString(),  fromSpeedSystem ) );
 	sTmp = data.value("Wind Direction").toString();
 	if( d->checkStringValue( sTmp) )
 		weather.setWindDirection( sTmp );
+	if (data.contains("Windchill"))
+		weather.setWindChill(data.value("Windchill").toInt());
+	if (data.contains("Wind Gust"))
+		weather.setWindGust( d->convertSpeed( data.value("Wind Gust").toString(),  fromSpeedSystem ) );
+	
+	weather.setWindShortText( d->createWindShortText(weather) );
+	weather.setPressureShortText( d->createPressureText(weather) );
 
 	sTmp = data.value("Current Conditions").toString();
 	if( d->checkStringValue( sTmp ) )
@@ -372,8 +383,10 @@ WeatherDataProcessor::updateLocation( CityWeather & cityInfo,
 		{
 			break;
 		}
-		else if( pDay->date() == yesterday && pDay->sunrise().isValid() &&
-		    QTime::currentTime() < pDay->sunrise() )
+		else if( pDay->date() == yesterday &&
+		         pDay->hasNightValues() &&
+		         pDay->sunrise().isValid() &&
+		         QTime::currentTime() < pDay->sunrise() )
 		{
 			break;
 		}
@@ -513,13 +526,8 @@ WeatherDataProcessor::loadData( CityWeather & cityInfo ) const
 void
 WeatherDataProcessor::createDetailsPropertyMap( const QList<Yawp::DetailsProperty> & vProperties )
 {
-	d->vDetailsPropertyMap.clear();
-	QList<Yawp::DetailsProperty>::const_iterator itList = vProperties.begin();
-	short iPos(0);
-	for( ; itList != vProperties.end(); ++itList, ++iPos )
-	{
-		d->vDetailsPropertyMap[ *itList ] = iPos;
-	}
+	d->vDetailsProperty.clear();
+	d->vDetailsProperty = vProperties;
 }
 
 bool
@@ -565,6 +573,7 @@ WeatherDataProcessor::setForecastExtraValues(
 		weather.setTemperatureRealFeelHigh( d->convertTemp(vForecast.at(6), fromTempSystem) );
 		weather.setTemperatureRealFeelLow(  d->convertTemp(vForecast.at(7), fromTempSystem) );
 
+		weather.setWindShortText( d->createWindShortText(weather) );
 		bReturn = true;
 	}
 	return bReturn;
@@ -584,6 +593,29 @@ WeatherDataProcessor::setForecastSun( YawpDay & day, const QString & sValue ) co
 	return false;
 }
 
+/* The string for sunrise and sunset looks like this: "Monday December 12, 2011 at 15:57"
+ * IONs will return a english text always, no matter, which languages has been chosen.
+ * In KDE 4.7.2 - I noticed KDE is not able to parse this string, when languages is not english!?
+ * Therefore I extract the time from the string and parse this part only.
+ */
+QTime
+WeatherDataProcessor::Private::getTime(const QString & dateTime) const
+{
+	QTime time;
+	int startPos = dateTime.lastIndexOf("at");
+	if (startPos > 0)
+	{
+		startPos += 3;
+		int endPos = dateTime.indexOf(' ', startPos + 1);
+		if (endPos < 0)
+			endPos = dateTime.length();
+		
+		QString timeString = dateTime.mid(startPos, endPos - startPos);
+		time = QTime::fromString(timeString, "h:mm");
+	}
+	return time;
+}
+
 void
 WeatherDataProcessor::Private::setUVValues( YawpWeather & weather,
                                             const QString & sUVIndex,
@@ -624,13 +656,55 @@ WeatherDataProcessor::Private::checkStringValue( const QString & value ) const
 	return true;
 }
 
+QString
+WeatherDataProcessor::Private::createWindShortText(const YawpWeather & weather) const
+{
+	QString sText;
+	if( weather.windSpeed() < SHRT_MAX )
+	{
+		sText.append( QString("%1 %2 ")
+			.arg( weather.windSpeed() )
+			.arg( Utils::GetUnitString( speedSystem ) ) );
+		sText.append( i18nc("Wind direction", weather.windDirection().toUtf8().constData()) );
+	}
+	return sText;
+}
+
+QString
+WeatherDataProcessor::Private::createPressureText(const YawpWeather & weather) const
+{
+	QString sText;
+	if( weather.pressure() < SHRT_MAX )
+	{
+		sText.append( QString("\n%1 %2")
+			.arg( weather.pressure() )
+			.arg( Utils::GetUnitString(pressureSystem) ));
+
+		if( !weather.pressureTendency().isEmpty() )
+		{
+			// Pressure tendency arrow
+			QChar sPressureSymbol;
+			QChar sPressureShort = weather.pressureTendency()[0].toLower();
+			if (sPressureShort == QChar('s'))
+				sPressureSymbol = QChar(0x279C); // steady
+			if (sPressureShort == QChar('r') || sPressureShort == QChar('i'))
+				sPressureSymbol = QChar(0x279A); // rising
+			if (sPressureShort == QChar('f') || sPressureShort == QChar('d') )
+				sPressureSymbol = QChar(0x2798); // falling
+			
+			sText.append( QString(" %1").arg(sPressureSymbol) );
+		}
+	}
+	return sText;
+}
+
 float
 WeatherDataProcessor::Private::parseFloat( const QString & value ) const
 {
 	if( !checkStringValue( value ) )
 		return FLT_MAX;
 
-	bool bOk(false);
+	bool bOk = false;
 	float fValue = value.toFloat( &bOk );
 	if( bOk )
 		return fValue;
@@ -638,8 +712,7 @@ WeatherDataProcessor::Private::parseFloat( const QString & value ) const
 }
 
 short
-WeatherDataProcessor::Private::convertDistance( const QString & value,
-												YAWP_DISTANCE_UNIT fromSystem ) const
+WeatherDataProcessor::Private::convertDistance( const QString & value, YAWP_DISTANCE_UNIT fromSystem ) const
 {
 	float fValue = parseFloat( value );
 	if( fValue == FLT_MAX )
@@ -660,8 +733,7 @@ WeatherDataProcessor::Private::convertDistance( const QString & value,
 }
 
 short
-WeatherDataProcessor::Private::convertPressure( const QString & value,
-												YAWP_PRESSURE_UNIT fromSystem ) const
+WeatherDataProcessor::Private::convertPressure( const QString & value, YAWP_PRESSURE_UNIT fromSystem ) const
 {
 	float fValue = parseFloat( value );
 	if( fValue == FLT_MAX )
@@ -682,8 +754,7 @@ WeatherDataProcessor::Private::convertPressure( const QString & value,
 
 
 short
-WeatherDataProcessor::Private::convertTemp( const QString & value,
-											YAWP_TEMPERATURE_UNIT fromSystem ) const
+WeatherDataProcessor::Private::convertTemp( const QString & value, YAWP_TEMPERATURE_UNIT fromSystem ) const
 {
 	float fValue = parseFloat( value );
 	if( fValue == FLT_MAX )
@@ -704,8 +775,7 @@ WeatherDataProcessor::Private::convertTemp( const QString & value,
 }
 
 short
-WeatherDataProcessor::Private::convertSpeed( const QString & value,
-											 YAWP_SPEED_UNIT fromSystem ) const
+WeatherDataProcessor::Private::convertSpeed( const QString & value, YAWP_SPEED_UNIT fromSystem ) const
 {
 	float fValue = parseFloat( value );
 	if( fValue == FLT_MAX )
@@ -853,85 +923,66 @@ WeatherDataProcessor::Private::initIconMap(const QString & resource)
 void
 WeatherDataProcessor::Private::createPropertyStringList( const YawpDay & day, YawpWeather & weather ) const
 {
-	QMap<short, Yawp::DetailsProperty> vPropertyMap;
+	Plasma::Theme * theme = Plasma::Theme::defaultTheme();
+	QFont font = theme->font(Plasma::Theme::DefaultFont);
 
-	//--- check for properties and add them to the list ---
-	if( !weather.description().isEmpty() )
-		addPropertyItem( vPropertyMap, Yawp::WeatherDescription );
-	if( weather.temperatureRealFeelHigh() < SHRT_MAX && weather.temperatureRealFeelLow() < SHRT_MAX )
-		addPropertyItem( vPropertyMap, Yawp::RealfeelTemperature );
-	if( weather.dewpoint() < SHRT_MAX )
-		addPropertyItem( vPropertyMap, Yawp::Dewpoint );
-	if( weather.pressure() < SHRT_MAX )
-		addPropertyItem( vPropertyMap, Yawp::Pressure );
-	if( weather.uvIndex() < SHRT_MAX )
-		addPropertyItem( vPropertyMap, Yawp::UV );
-	if( weather.visibility() < SHRT_MAX )
-		addPropertyItem( vPropertyMap, Yawp::Visibility );
-	if( day.sunrise().isValid() && day.sunset().isValid() )
-		addPropertyItem( vPropertyMap, Yawp::SunriseSunset );
-
-	QFont font( KGlobalSettings::generalFont() );
 	font.setBold(false);
 	font.setPixelSize(15);
 	QTextLayout textLayout;
 	textLayout.setFont(font);
-
-	QMap<short, Yawp::DetailsProperty>::const_iterator itMap = vPropertyMap.begin();
-	QString sText;
-	QString sOffsetText(2, QChar(' '));
+	
+	QString sOffsetText(3, QChar(' '));
 	QFontMetrics fm(font);
-	qreal dOffsetWidth( (qreal)fm.width(sOffsetText) );
-
-	for( ; itMap != vPropertyMap.end(); ++itMap )
+	qreal dOffsetWidth = (qreal)fm.width(sOffsetText);
+	
+	//  go though all property items that has been specified to show in details area
+	//
+	QList<Yawp::DetailsProperty>::const_iterator itMap = vDetailsProperty.begin();
+	for( ; itMap != vDetailsProperty.end(); itMap++ )
 	{
-		sText = createPropertyText(*itMap, day, weather);
-
-/**** Just for debug to test the linebreak stuff. When a text needs to be wrapped around several lines.
-      than we will indent all lines expect the first one. This way it is easy to see when a new property text
-      starts.
-		switch( qrand() % 5 )
-		{
-			case 0: break;
-			case 1: case 4:	sText.append(" A little bit more text to test the line-break."); break;
-			case 2: sText.append(" xyz abc"); break;
-			case 3: sText.append(" A short text at the end."); break;
-		}
-*/
+		//--- Create property text for requested property ---
+		QString sText = createPropertyText(*itMap, day, weather);
+		
+		//--- When propery text is empty, continue with next property ---
+		if( sText.isEmpty() )
+			continue;
+	
+		/*   When a text needs to be wrapped around several lines.
+		 *   than we will indent all lines expect the first one.
+		 *   This way it is easy to see when a new property text starts.
+		 */
 		sText.replace('\n', QChar::LineSeparator);
 		textLayout.setText( sText );
-		QSizeF textSize = doTextLayout( textLayout, MAX_PROPERTY_TEXT_WIDTH, dOffsetWidth);
-		if( textSize.width() > MAX_PROPERTY_TEXT_WIDTH )
+		doTextLayout( textLayout, MAX_PROPERTY_TEXT_WIDTH, dOffsetWidth);
+		
+		/*   Add the propery text lines for current property to property-text-line-list
+		 */
+		for( int iLine = 0; iLine < textLayout.lineCount(); iLine++ )
 		{
-			qreal dTextLength(0.0f);
-			for( int i=0; i<textLayout.lineCount(); ++i )
-				dTextLength += (i+1 == textLayout.lineCount() ? MAX_PROPERTY_TEXT_WIDTH : textLayout.lineAt(i).naturalTextWidth());
-			QFontMetrics fm(font);
-			sText = fm.elidedText(sText, Qt::ElideRight, dTextLength);
-			textLayout.setText( sText );
-			textSize = doTextLayout( textLayout, MAX_PROPERTY_TEXT_WIDTH, dOffsetWidth );
-		}
+			qreal dMaxLineWidth = (iLine == 0 ? MAX_PROPERTY_TEXT_WIDTH : MAX_PROPERTY_TEXT_WIDTH-dOffsetWidth);
 
-		for( int i=0; i<textLayout.lineCount(); ++i )
-		{
-			QTextLine line( textLayout.lineAt(i) );
-			int iTS = line.textStart();
-			int iTW = line.textLength();
+			QTextLine line = textLayout.lineAt(iLine);
+			int iStart = line.textStart();
+			int iWidth = line.textLength();
 
-			QChar::Category category( textLayout.text().at(iTS+iTW-1).category() );
-			iTW += (category == QChar::Separator_Line || category == QChar::Separator_Space ? -1 : 0);
-//			dDebug() << line.textStart() << line.textLength() << textLayout.text().mid( iTS, iTW ) << "   " << line.naturalTextWidth();
-			if( i==0 )
-				weather.propertyTextLines().append( textLayout.text().mid( iTS, iTW ) );
+			QChar::Category category = textLayout.text().at(iStart+iWidth-1).category();
+			iWidth += (category == QChar::Separator_Line || category == QChar::Separator_Space ? -1 : 0);
+
+			// When text line does not fit we have to trim textline
+			// methode doTextLayout will abort text layout when one line does not fit
+			QString sText = textLayout.text().mid( iStart, iWidth );
+			if( line.naturalTextWidth() > dMaxLineWidth )
+				sText =  fm.elidedText(sText, Qt::ElideRight, dMaxLineWidth);
+			
+			//--- Finally, append the text line to the property text lines ---
+			if( iLine == 0 )
+				weather.propertyTextLines().append( sText );
 			else
-				weather.propertyTextLines().append( sOffsetText + textLayout.text().mid( iTS, iTW ) );
+				weather.propertyTextLines().append( sOffsetText + sText );
 		}
-
-//		dDebug() << "--------------------------------------------";
 	}
 }
 
-
 inline QString
 WeatherDataProcessor::mapConditionIcon( const QString & iconName ) const
 {
@@ -943,68 +994,131 @@ WeatherDataProcessor::Private::createPropertyText( const Yawp::DetailsProperty p
                                                    const YawpDay & day,
                                                    const YawpWeather & weather ) const
 {
+	QString sText;
+  
 	switch( property )
 	{
 	case Yawp::Dewpoint:
-		return i18n("Dewpoint: %1", weather.dewpoint())	+ QChar(0xB0);
+		if( weather.dewpoint() < SHRT_MAX )
+			return i18n("Dewpoint: %1", weather.dewpoint())	+ QChar(0xB0);
 		break;
 
 	case Yawp::Pressure:
+		if( weather.pressure() < SHRT_MAX )
 		{
-			QString sTmp( i18n("Pressure: %1 %2",
+			sText = i18n("Pressure: %1 %2",
 				weather.pressure(),
-				Utils::GetUnitString(pressureSystem)) );
+				Utils::GetUnitString(pressureSystem));
 
 			if( weather.pressureTendency() != "Unavailable" &&
 			    !weather.pressureTendency().isEmpty() )
 			{
-				sTmp.append( QString("  %1")
+				sText.append( QString("  %1")
 					.arg( i18nc("Pressure", weather.pressureTendency().toUtf8().constData())) );
 			}
-			return sTmp;
 		}
 		break;
 
-		case Yawp::RealfeelTemperature:
+	case Yawp::RealfeelTemperature:
 		{
+			int iTempCount = 0;
+			
 			QString sHigh(i18n("N/A"));
 			QString sLow(i18n("N/A"));
+			
 			if( weather.temperatureRealFeelHigh() < SHRT_MAX )
+			{
 				sHigh = QString::number(weather.temperatureRealFeelHigh()) + QChar(0xB0);
+				iTempCount += 1;
+			}
 			if( weather.temperatureRealFeelLow() < SHRT_MAX )
+			{
 				sLow = QString::number(weather.temperatureRealFeelLow()) + QChar(0xB0);
-			return i18n("Real feel: %1 / %2", sHigh, sLow );
+				iTempCount += 1;
+			}
+			
+			//--- Return the string only when we have at least one valid temperature ---
+			if( iTempCount > 0 )
+				return i18n("Real feel: %1 / %2", sHigh, sLow );
 		}
 		break;
 
-		case Yawp::SunriseSunset:
+	case Yawp::SunriseSunset:
 		{
+			int iTempCount = 0;
+			
 			QString sSunrise(i18n("N/A"));
 			QString sSunset(i18n("N/A"));
+			
 			if( day.sunrise().isValid() )
+			{
 				sSunrise = KGlobal::locale()->formatTime(day.sunrise());
+				iTempCount += 1;
+			}
 			if( day.sunset().isValid() )
+			{
 				sSunset = KGlobal::locale()->formatTime(day.sunset());
-			return i18n("Sunrise at %1, sunset at %2", sSunrise, sSunset);
+				iTempCount += 1;
+			}
+			if( iTempCount > 0 )
+				return i18n("Sunrise at %1, sunset at %2", sSunrise, sSunset);
 		}
 		break;
 
-		case Yawp::UV:
-			return i18n("UV Index: %1  %2",
+	case Yawp::UV:
+		if( weather.uvIndex() < SHRT_MAX )
+		{
+			return i18n("UV Index: %1 %2",
 				weather.uvIndex(), weather.uvRating());
-			break;
+		}
+		break;
 
-		case Yawp::Visibility:
+	case Yawp::Visibility:
+		if( weather.visibility() < SHRT_MAX )
+		{
 			return i18n("Visibility: %1 %2",
 				weather.visibility(),
 				Utils::GetUnitString( distanceSystem ));
-				break;
+		}
+		break;
 
-		case Yawp::WeatherDescription:
-			return Utils::LocalizedWeatherString(weather.description());
-			break;
+	case Yawp::WeatherDescription:
+		if( !weather.description().isEmpty() )
+		{
+			// weather condition from Wunderground contains >&deg;<
+			// for the degree-character,
+			// Utils::LocalizedWeatherString will replace 'C - Celsius' and 
+			// 'F - Fahrenheit' to lower capitals
+			// so we replace both of them after string has been translated
+			// through Utils::LocalizedWeatherString as NASTY WORKAROUND
+			sText = Utils::LocalizedWeatherString(weather.description());
+			int iPos = 0;
+
+			while( iPos >= 0 )
+			{
+				iPos = sText.indexOf("&deg;", iPos);
+				if( iPos > 0 )
+				{
+					sText.replace(iPos, 5, QChar(0xB0));
+
+					// Utils::LocalizedWeatherString is adding a space-character
+					// between degree-character and the temperature unit sign (C/F)
+					// this is the reason, why we replace two characters
+					if( sText.mid(iPos+1, 1).compare(" ") == 0 )
+						sText.remove(iPos+1, 1);
+					
+					QString sTemp = sText.mid(iPos+1, 1);
+					if( sTemp.compare("f", Qt::CaseSensitive) == 0)
+						sText.replace(iPos+1, 1, 'F');
+					else if( sTemp.compare("c", Qt::CaseSensitive) == 0 )
+						sText.replace(iPos+1, 1, 'C');
+				}
+				iPos = (iPos+1 < sText.length() && iPos >= 0 ? iPos+1 : -1);
+			}
+		}
+		break;
 	}
-	return QString();
+	return sText;
 }
 
 QSizeF
@@ -1021,7 +1135,6 @@ WeatherDataProcessor::Private::doTextLayout( QTextLayout & textLayout, qreal dMa
 		if (!line.isValid())
 			break;
 
-//		dDebug() << "TextLayout: " << textLayout.lineCount();
 		line.setLineWidth(dMaxWidth - (textLayout.lineCount()<=1 ? 0.0f : dOffset ));
 		line.setPosition(QPointF(0, dHeight));
 		dHeight += line.height();
